#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "chump.h"
#include "dotparse.h"
#include "main.h"



void
on_ok_button1_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  chump_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data));
  on_reload_clicked(NULL,NULL);
}


void
on_reload_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  GList* toplist;
  GList* cpulist;
  GScanner *scanner;
  PtrSCANNode Scantopnode;
  DefinitionStack* isa_list;

  GtkWidget *menuitem;
  GtkWidget* optionmenu_menu = gtk_menu_new();

  asm_tables = NULL;              // Clear asm tables
  
  if (chump_filename){
    scanner = ScanOpenSCANFile (chump_filename);              // open file
    g_print("Loading:%s\n",chump_filename);
    }
    
  else {
    scanner = ScanOpenSCANString (chump_string);
    g_print("Loading internal chump file\n");
    }
        
  if (!scanner) return;
  Scantopnode = ScanParseSCANNode (scanner, FALSE);   // parse it
  toplist=Scantopnode->body.list;                     // Get top level node
  cpulist=Scantopnode->body.list;                     // Get top level node
  
  while((cpulist = ScanfindSymbolList    (toplist,"isa",NULL))){
        
        g_print("Loaded: %s\n",SCAN_SYMBOL(SCAN_LIST_FIRST(cpulist)));
        asm_tables = asm_define(cpulist, asm_tables, SCAN_SYMBOL(SCAN_LIST_FIRST(cpulist)));      // Parse the desciption
        toplist = ScanfindSymbolListNext(toplist,"isa",NULL);       // Move on
    }
  
  isa_list = asm_tables;                    //Walk though the ISAs
  while (isa_list){
    menuitem = gtk_menu_item_new_with_label (isa_list->string);     //Make a new menu
    gtk_widget_show (menuitem);
    gtk_menu_prepend (GTK_MENU (optionmenu_menu), menuitem);        //Add ISAs to menu
    gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
                      GTK_SIGNAL_FUNC (callback_memwindow_isa),(gpointer)isa_list);     // When pressed call callback_memwindow_isa
    current_isa = isa_list->rules;
    isa_list=isa_list->next;
    }
    
  gtk_option_menu_set_menu (GTK_OPTION_MENU(gtk_object_get_data (GTK_OBJECT(main_window),"isa_selection")),optionmenu_menu);
                                                                    //Assign new menu list
  
}


void callback_memwindow_isa (GtkMenuItem* menuitem, gpointer newvalue) /* the line width and gran */
{
  current_isa = ((DefinitionStack*) newvalue)->rules;
  g_print("Now using: %s\n",((DefinitionStack*) newvalue)->string);
}


void
on_assembly_entry_activate             (GtkEditable     *editable,
                                        gpointer         user_data)
{
  char* bytes;
  char* text;
  int temp;
  Asm* asmed;
  GList* asmlist;
  
  text = gtk_editable_get_chars(editable,0,-1);      // Get entry
  temp = view_hexstr2int(gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data (GTK_OBJECT(main_window),"address_entry")),0,-1));
  asmlist = asm_assemble(text,current_isa, temp,1);                    // Assamble
  if (!asmlist){
        g_free(text);
        gtk_entry_set_text (GTK_ENTRY(gtk_object_get_data (GTK_OBJECT(main_window),"hex_entry")),"Error");
        return;}
  
  asmed = (Asm*) (asmlist->data);
  if (asmed->binary == &dis_error)   {
        g_free(text);
        gtk_entry_set_text (GTK_ENTRY(gtk_object_get_data (GTK_OBJECT(main_window),"hex_entry")),"Error");
        return;
        }
  g_free (text);
  temp = asm_sbprint(asmed->binary,(uchar**) &bytes);                         // Get hex
  text = view_chararr2hexstrbe(temp, bytes);
  gtk_entry_set_text (GTK_ENTRY(gtk_object_get_data (GTK_OBJECT(main_window),"hex_entry")),text);
  g_print("Assembled to %s\n",text);
  g_free(text);
  g_free(bytes);
  
  while(asmlist) {
    asm_freestringlist(((Asm*)asmlist->data)->binary);
    g_free(asmlist->data);
    asmlist = g_list_remove(asmlist,asmlist->data);
    }
}


void
on_hex_entry_activate                  (GtkEditable     *editable,
                                        gpointer         user_data)
{

 Disasm disasm;
 int temp;
 temp = view_hexstr2int(gtk_editable_get_chars(GTK_EDITABLE(gtk_object_get_data (GTK_OBJECT(main_window),"address_entry")),0,-1));
 disasm.bitfield = view_hexstr2glist(gtk_editable_get_chars(editable,0,-1));
 disasm  = asm_disassemble (disasm.bitfield, current_isa, temp);      // Get entry
 if (disasm.distext != &dis_error ){
    gtk_entry_set_text (GTK_ENTRY(gtk_object_get_data (GTK_OBJECT(main_window),"assembly_entry"))
                        ,asm_sprint(disasm.distext, g_strdup("")));
    g_print("%s\n",asm_sprint(disasm.distext, g_strdup("")));       //
    asm_freestringlist (disasm.distext);
    }
 else gtk_entry_set_text (GTK_ENTRY(gtk_object_get_data (GTK_OBJECT(main_window),"assembly_entry")),"Error");


}



char* view_chararr2hexstrbe(int count, uchar* values) 
{
  char* ret = g_new(char, count * 2 + 1);  /* convert bit array into hex string*/
  char* ptr = ret;                         /* bigendian*/
  while (count--){
    g_snprintf(ptr,3,"%02X ",(int) values[count]);
    ptr+=2;
    }
  return ret;                              /* Must g_free result*/
}


GList* view_hexstr2glist(char* text)
{
  char c;
  GList* ret=NULL;
  int count;
  
  while (*text){
		c = *text;
        if (c>='0' && c<='9') c=c-'0';
        else if (c>='a' && c<='f') c=c-'a'+10;
        else if (c>='A' && c<='F') c=c-'A'+10;
        else return NULL;
        
		for (count = 4; count; count--)
		{
			c <<= 1;
			ret = g_list_prepend (ret, GINT_TO_POINTER ((c & 16)>>4));
		}
    text++;
    }
  return ret;                              /* Must g_free result*/
}


int view_hexstr2int(char* text)
{
  char c;
  int ret=0;
  
  while (*text){
		c = *text;
        if (c>='0' && c<='9') c=c-'0';
        else if (c>='a' && c<='f') c=c-'a'+10;
        else if (c>='A' && c<='F') c=c-'A'+10;
        else return 0;
        ret=(ret<<4)+c;
        text++;
        }
  g_print("addres:%x\n",ret);
  return ret;                              /* Must g_free result*/
}

